-- EquipNSpecReminder Addons --
-- Create the addons in memory
local EquipNSpecReminder = LibStub("AceAddon-3.0"):NewAddon("EquipNSpecReminder", "AceEvent-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("EquipNSpecReminder")

local IsEquipChanged = false;
local doIgnoreEvent = false;

local OPTION_SELECT_WINDOW = "-- "..L["Show selection window"].." --";
local OPTION_DO_NOTHING = "-- "..L["Do nothing"].." --";
local ANNOUNCE_RAID = L["Raid Warning"];
local ANNOUNCE_CHAT = L["Chat Window"];
local ANNOUNCE_NO   = L["Do not announce"];


local myOptionsTable = {
   type = "group",
   get = function(item) return EquipNSpecReminder.db.profile[item[#item]] end,
   set = function(item, value) EquipNSpecReminder.db.profile[item[#item]] = value end,
   args = {
      headerEquip = {
         type = "header",
         name = L["Default equipment set chosen on spec change"],
         order = 1,
      },
      defaultEquipFirst = {
         type = "select",
         name = "Standard Equipment Set (First Spec)",
         desc = "Standard Equipment Set das beim Wechsel auf Spec1 verwendet wird",
         style = "dropdown",
         values = function()
            local EquipSets = {[OPTION_DO_NOTHING] = OPTION_DO_NOTHING, [OPTION_SELECT_WINDOW] = OPTION_SELECT_WINDOW}
            for i=1,GetNumEquipmentSets() do
                local name,_,_ = GetEquipmentSetInfo(i)
                EquipSets[name] = name
            end
            return EquipSets
         end,
         order = 2,
         width = "double",
      },
      blankDefaultEquipFirst = {
         type = "description",
         name = "",
         desc = "",
         order = 3,
      },
      defaultEquipSec = {
         type = "select",
         name = "Standard Equipment Set (Secondary Spec)",
         desc = "Standard Equipment Set das beim Wechsel auf Spec2 verwendet wird",
         style = "dropdown",
         values = function()
            local EquipSets = {[OPTION_DO_NOTHING] = OPTION_DO_NOTHING, [OPTION_SELECT_WINDOW] = OPTION_SELECT_WINDOW}
            for i=1,GetNumEquipmentSets() do
                local name,_,_ = GetEquipmentSetInfo(i)
                EquipSets[name] = name
            end
            return EquipSets
         end,
         order = 4,
         width = "double"
      },
      blankDefaultEquipSec = {
         type = "description",
         name = "",
         desc = "",
         order = 5,
      },      
      announce = {
         type = "select",
         name = L["Announce change in"],
         style = "dropdown",
         values = {[ANNOUNCE_NO] = ANNOUNCE_NO, [ANNOUNCE_RAID] = ANNOUNCE_RAID, [ANNOUNCE_CHAT] = ANNOUNCE_CHAT},
         order = 6, 
      },
      blankAnnounce = {
         type = "description",
         name = "",
         desc = "",
         order = 7,
      },
      headerSpec = {
         type = "header",
         name = L["Default Spec chosen on equipment set change"],
         order = 8,
      },      
   }
};





-------------------------------------------------------------------------------
-- Hooks and boostrap code
-------------------------------------------------------------------------------

-- We're alive!
function EquipNSpecReminder:OnInitialize()
   
   self.db = LibStub("AceDB-3.0"):New("EquipNSpecReminderDB");
   
   LibStub("AceConfig-3.0"):RegisterOptionsTable("EquipNSpecReminder", myOptionsTable, nil);
   LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EquipNSpecReminder", "EquipNSpecReminder")


--local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
--LibStub("AceConfig-3.0"):RegisterOptionsTable("AuctionLite Profiles", profiles);
  
end


function EquipNSpecReminder:OnEnable()
    -- Do more initialization here, that really enables the use of your addon.
    -- Register Events, Hook functions, Create Frames, Get information from
    -- the game that wasn't available in OnInitialize

    self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
    self:RegisterEvent("EQUIPMENT_SWAP_FINISHED");
    
    
    self:HookScript(UIParent, "OnUpdate", "OnMyUpdate")
    
    
    

    if theFrame == nil then
        theFrame = CreateFrame("frame","MyAddonFrame")
        theFrame:SetBackdrop({
            bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
            edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
            tile=1, tileSize=32, edgeSize=32,
            insets={left=11, right=12, top=12, bottom=11}
        })
        theFrame:SetWidth(400)
        theFrame:SetHeight(145)
        theFrame:SetPoint("CENTER",UIParent)
        theFrame:EnableMouse(true)
        theFrame:SetMovable(true)
        theFrame:RegisterForDrag("LeftButton")

        theFrame:SetScript("OnDragStart", function(self) self:StartMoving() end)
        theFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
        theFrame:SetFrameStrata("FULLSCREEN_DIALOG")

        theFrame.IntroLabel=theFrame:CreateFontString(nil,"OVERLAY","GameTooltipText");
        theFrame.IntroLabel:SetPoint("TOP",theFrame,"TOP",0,-10);
        theFrame.IntroLabel:SetHeight(60);
        theFrame.IntroLabel:SetWidth(300);
        theFrame.IntroLabel:SetJustifyH("CENTER");
        theFrame.IntroLabel:SetTextColor(1,0,0);

        theFrame.EquipData = CreateFrame("Frame", "EquipDropDown", theFrame, "UIDropDownMenuTemplate");
        theFrame.EquipData:SetPoint("TOP",theFrame.IntroLabel,"BOTTOM",0,0);
        UIDropDownMenu_SetWidth(theFrame.EquipData,220);
        UIDropDownMenu_SetButtonWidth(theFrame.EquipData, 20);
        UIDropDownMenu_Initialize(theFrame.EquipData, DropdownData_Initialise);

        theFrame.OkButton = CreateFrame("button","MyAddonButtonOk", theFrame, "UIPanelButtonTemplate")
        theFrame.OkButton:SetHeight(24)
        theFrame.OkButton:SetWidth(90)
        theFrame.OkButton:SetPoint("BOTTOM", theFrame, "BOTTOM", -60, 10)
        theFrame.OkButton:SetText(OKAY)
        theFrame.OkButton:SetScript("OnClick", function(self) AcceptChange() end)

        theFrame.CancelButton = CreateFrame("button","MyAddonButtonCancel", theFrame, "UIPanelButtonTemplate")
        theFrame.CancelButton:SetHeight(24)
        theFrame.CancelButton:SetWidth(90)
        theFrame.CancelButton:SetPoint("BOTTOM", theFrame, "BOTTOM", 60, 10)
        theFrame.CancelButton:SetText(CANCEL)
        theFrame.CancelButton:SetScript("OnClick", function(self) self:GetParent():Hide() end)

        theFrame:Hide()
    end
end

function EquipNSpecReminder:OnDisable()
  -- Unhook, Unregister Events, Hide frames that you created.
  -- You would probably only use an OnDisable if you want to
  -- build a "standby" mode, or be able to toggle modules on/off.
end






-- Timeout function
function EquipNSpecReminder:OnMyUpdate(arg1, time) -- "time" is in secs so may me small like 0.020s
   if equipAutoChanged == true then
      timeElapsed = timeElapsed + time;
   
      if(timeElapsed >= 3) then
         timeElapsed = 0;
         equipAutoChanged = false;
         RaidNotice_AddMessage(RaidWarningFrame, "Automatic Equipment change could not be performed!", ChatTypeInfo["RAID_WARNING"],10)
      end
   else
      timeElapsed = 0;
   end
end




function EquipNSpecReminder:EQUIPMENT_SWAP_FINISHED(_,success, name)
   if equipAutoChanged == false then
      if doIgnoreEvent == false and success and GetNumTalentGroups() >= 1 then
         IsEquipChanged = true;
         theFrame.IntroLabel:SetText(L["Please also change your spec."].."\n\n"..L["Choose spec:"]);
         if GetActiveTalentGroup()==1 then
            UIDropDownMenu_SetSelectedValue(theFrame.EquipData, 2);
         else
            UIDropDownMenu_SetSelectedValue(theFrame.EquipData, 1);
         end
   
         theFrame:Show()
      end
   else
      -- TODO: only announce confirmation when automatic change has really happened
      -- RaidNotice_AddMessage(RaidWarningFrame, "Automatically changed to equipment set \"".. sdfgdfgdfg.."\"", ChatTypeInfo["RAID_WARNING"],5)      
   end
   equipAutoChanged = false;
   doIgnoreEvent = false;
end



function EquipNSpecReminder:ACTIVE_TALENT_GROUP_CHANGED()
   if doIgnoreEvent == false and GetNumEquipmentSets() ~= 0 then
      IsEquipChanged = false;
      
      -- check if there was a change to the primary spec and if a 
      -- default equipment is defined
      if GetActiveTalentGroup()==1 then
         if self.db.profile.defaultEquipFirst == OPTION_DO_NOTHING then
            -- do nothing
         elseif self.db.profile.defaultEquipFirst == OPTION_SELECT_WINDOW then
            -- show selection frame
            self:ShowEquipSelectFrame();         
         else
            -- check if equipment set exists
            icon, lessIndex = GetEquipmentSetInfoByName(self.db.profile.defaultEquipFirst)
            if icon ~= nil then 
               equipAutoChanged = true;
               UseEquipmentSet(self.db.profile.defaultEquipFirst)
               self:AnnounceChange (L["equipment set"], self.db.profile.defaultEquipFirst)
            else
               -- Equipment set does not exist. Delete wrong config
               self.db.profile.defaultEquipFirst = nil
            end
         end

      -- check if there was a change to the secondary spec and if a 
      -- default equipment is defined      
      elseif GetActiveTalentGroup()==2 then
         if self.db.profile.defaultEquipSec == OPTION_DO_NOTHING then
            -- do nothing
         elseif self.db.profile.defaultEquipSec == OPTION_SELECT_WINDOW then
            -- show selection frame
            self:ShowEquipSelectFrame();   
         else   
            -- check if equipment set exists
            icon, lessIndex = GetEquipmentSetInfoByName(self.db.profile.defaultEquipSec)
            if icon ~= nil then
               equipAutoChanged = true;
               UseEquipmentSet(self.db.profile.defaultEquipSec)
               self:AnnounceChange (L["equipment set"], self.db.profile.defaultEquipSec)
            else
               -- Equipment set does not exist. Delete wrong config
               self.db.profile.defaultEquipSec = nil
            end     
         end      
      end        
   end
   doIgnoreEvent = false;
end


function EquipNSpecReminder:AnnounceChange (what, toWhat)
   local announcmentText = "Automatically changed to "..L["equipment set"].." \"".. toWhat .."\"";
   if self.db.profile.announce == ANNOUNCE_RAID then
      RaidNotice_AddMessage(RaidWarningFrame, announcmentText, ChatTypeInfo["RAID_WARNING"],5)
   elseif self.db.profile.announce == ANNOUNCE_CHAT then
      print (announcmentText);
   else
      -- do nothing
   end
end



-- show frame to select an equipment set
function EquipNSpecReminder:ShowEquipSelectFrame ()
   -- show selection frame
   theFrame.IntroLabel:SetText(L["Please also change equipment."].."\n\n"..L["Choose set:"]);
   local name,_,_ = GetEquipmentSetInfo(1);
   UIDropDownMenu_SetSelectedValue(theFrame.EquipData, name);
   _G[theFrame.EquipData:GetName().."Text"]:SetText(name);
   theFrame:Show()
end            


-- OnClick event for dropdown select item
function DropdownData_OnClick(item)
  UIDropDownMenu_SetSelectedValue(theFrame.EquipData, item.value);
end

function AcceptChange()
    doIgnoreEvent = true;
    if IsEquipChanged then
        SetActiveTalentGroup(UIDropDownMenu_GetSelectedValue(theFrame.EquipData));
    else
        equipAutoChanged = true;
        UseEquipmentSet(UIDropDownMenu_GetSelectedValue(theFrame.EquipData));
    end
    theFrame:Hide()
end

function DropdownData_Initialise()
    if IsEquipChanged then
        local info = UIDropDownMenu_CreateInfo();

        for i=1,GetNumTalentGroups() do
            local talentpoints = "";
            local names = "";
            for k=1,GetNumTalentTabs() do
                id, name, description, iconTexture, pointsSpent, background, previewPointsSpent, isUnlocked = GetTalentTabInfo (k,false,false,i)
                if k>1 then
                    talentpoints = talentpoints.."/";
                    names = names.."/";
                end
                talentpoints = talentpoints..pointsSpent;
                names = names..name;
            end

            info.text = talentpoints.." ("..names..")"; --the text of the menu item
            info.func = DropdownData_OnClick; --sets the function to execute when this item is clicked
            info.value = i; -- the value of the menu item. This can be a string also.
            info.checked = nil; --initially set the menu item to being unchecked with a yellow tick
            UIDropDownMenu_AddButton(info, level); --Adds the new button to the drop down menu specified in the UIDropDownMenu_Initialise function. In this case, it's MyDropDownMenu
        end
    else
        if GetNumEquipmentSets() ~= 0 then
            local info = UIDropDownMenu_CreateInfo();

            for i=1,GetNumEquipmentSets() do
                local name,_,_ = GetEquipmentSetInfo(i)
                info.text = name; --the text of the menu item
                info.func = DropdownData_OnClick; --sets the function to execute when this item is clicked
                info.value = name; -- the value of the menu item. This can be a string also.
                info.checked = nil; --initially set the menu item to being unchecked with a yellow tick
                UIDropDownMenu_AddButton(info, level); --Adds the new button to the drop down menu specified in the UIDropDownMenu_Initialise function. In this case, it's MyDropDownMenu
            end
        end
    end
end
